from typing import Optional

from mixins import EqHashMixin

from pyexlatex.texgen import _usepackage_str
from pyexlatex.models.item import ItemBase


class Package(ItemBase):
    r"""
    Represents LaTeX \usepackage{}, pass to Document if any custom LaTeX packages are needed.
    """
    equal_attrs = ["name", "modifier_str"]
    __hash__ = EqHashMixin.__hash__

    def __init__(
        self, name: str, modifier_str: Optional[str] = None, eq_on_modifier: bool = True
    ):
        r"""
        :param name: Name of LaTeX package
        :param modifier_str: any options to be passed to the LaTeX package. E.g. if 'abc=true'
            is passed, it will render as \usepackage[abc=true]{mypackage}
        :param eq_on_modifier: Whether to consider any package options when checking equality
            of the package. Useful to set it to False when this package should not be added
            if it was previously added with different options
        """
        self.name = name
        self.modifier_str = modifier_str
        self.eq_on_modifier = eq_on_modifier

    def __repr__(self):
        return f"<Package(name={self.name})>"

    def __str__(self):
        return _usepackage_str(self.name, self.modifier_str)

    def __eq__(self, other):
        try:
            eq_on_modifier = self.eq_on_modifier and other.eq_on_modifier
        except AttributeError:
            return False
        if not eq_on_modifier:
            try:
                return self.name == other.name
            except AttributeError:
                return False

        # Compare using modifier
        try:
            return all(
                [self.name == other.name, self.modifier_str == other.modifier_str]
            )
        except AttributeError:
            return False

    def matches_name(self, other):
        return self.name == other



import pyexlatex as pl
from pyexlatex.texgen.packages.default import default_packages
from tests.base import GENERATED_FILES_DIR, INPUT_FILES_DIR
from tests.utils.pdf import compare_pdfs


def test_package():
    package = pl.Package('hyperref', modifier_str='hidelinks')
    assert str(package) == '\\usepackage[hidelinks]{hyperref}'


def test_package_conflict():
    section = pl.Section(['woo'], title='Section')
    section.init_data()
    section.add_package(pl.Package('hyperref', modifier_str='hidelinks'))
    # Without eq_on_modifier=False, raises error for package options conflict
    section.add_package(pl.Package('hyperref', modifier_str='linktoc=all', eq_on_modifier=False))
    doc = pl.Document([section])
    doc.to_pdf(GENERATED_FILES_DIR, outname='package conflict document')
    compare_pdfs(INPUT_FILES_DIR / 'package conflict document.pdf', GENERATED_FILES_DIR / 'package conflict document.pdf')


def test_package_conflict_with_document_packages():
    section = pl.Section(['woo'], title='Section')
    section.init_data()
    # Without eq_on_modifier=False, raises error for package options conflict
    section.add_package(pl.Package('hyperref', modifier_str='linktoc=all', eq_on_modifier=False))

    doc = pl.Document([section], packages=default_packages + [pl.Package('hyperref', modifier_str='hidelinks')])
    doc.to_pdf(GENERATED_FILES_DIR, outname='package conflict document')
    compare_pdfs(INPUT_FILES_DIR / 'package conflict document.pdf',
                 GENERATED_FILES_DIR / 'package conflict document.pdf')



from pyexlatex.models.caption import Caption
from tests.base import SimpleItemTest


class TestCaption(SimpleItemTest):
    item_class = Caption
    tag_name = "caption"


def test_short_caption():
    capt = Caption("woo yeah", short_caption="woo")
    assert str(capt) == "\\caption[woo]{woo yeah}"



from typing import Optional

from pyexlatex.models.item import SimpleItem
from pyexlatex.typing import PyexlatexItems


class Caption(SimpleItem):
    name = 'caption'

    def __init__(self, contents: PyexlatexItems, short_caption: Optional[str] = None):
        pre_modifiers: Optional[str] = None
        if short_caption is not None:
            pre_modifiers = self._wrap_with_bracket(short_caption)
        super().__init__(self.name, contents, pre_modifiers=pre_modifiers)


